use chrono::Local;
use serenity::all::{CommandDataOption, CommandDataOptionValue, CommandInteraction, Context, ResolvedOption, ResolvedValue};
use serenity::builder::{CreateCommand, CreateCommandOption, CreateEmbed, CreateEmbedAuthor, CreateEmbedFooter};
use serenity::model::application::CommandOptionType;

use crate::util::user_util;

pub async fn run(ctx: &Context, command: &CommandInteraction) -> CreateEmbed {
    let username = command.user.name.as_str();
    let current_time = Local::now().format("%Y-%m-%d @ %H:%M:%S");
    let options = &command.data.options;

    let query = if let Some(CommandDataOption {
                                value: CommandDataOptionValue::String(query), ..
                            }) = &options.first()
    {
        query
    } else {
        return CreateEmbed::new()
            .author(CreateEmbedAuthor::new("Rustendo"))
            .title("Error 400")
            .description("There is no query provied.")
            .footer(CreateEmbedFooter::new(format!(">{} | {}", current_time, username)))
    };

    let guild_id = match &command.guild_id {
        Some(guild_id) => guild_id,
        None => {
            return CreateEmbed::new()
                .author(CreateEmbedAuthor::new("Rustendo"))
                .title("GuildId not found")
                .description("Could not find guild id.")
                .footer(CreateEmbedFooter::new(format!("> {} | {}", current_time, username)));
        }
    };

    let (guild_id, channel_id) = {
        let guild = &ctx.cache.guild(guild_id).unwrap(); // TODO: This unwrap throws errors.
        // This may be unsafe, idk not sure yet
        let channel_id = guild
            .voice_states
            .get(&command.user.id)
            .and_then(|voice_state| voice_state.channel_id);
        (guild.id, channel_id)
    };

    let connect_to = match channel_id {
        Some(channel) => channel,
        None => {
            return CreateEmbed::new()
                .author(CreateEmbedAuthor::new("Rustendo"))
                .title("You are not in a VC.")
                .description("Join one to start playing music.")
                .footer(CreateEmbedFooter::new(format!("> {} | {}", current_time, username)));
        },
    };
    
    let manager = &songbird::get(ctx)
        .await
        .expect("Cannot get Songbird.")
        .clone();

    if !user_util::is_self_connected_to_vc(&ctx, &guild_id) {
        // self is connected to vc, check if user is in same vc
        let self_channel = user_util::get_self_vc_id(&ctx, &guild_id);
        // Check if user is in the same VC as the bot
        if self_channel != connect_to {
            return CreateEmbed::new()
                .author(CreateEmbedAuthor::new("Rustendo"))
                .title("You are not in my VC.")
                .description("Connect to my VC to control the music.")
                .footer(CreateEmbedFooter::new(format!("> {} | {}", current_time, username)));
        }
        // Connect to VC
        manager.join(guild_id, connect_to).await.expect("Cannot connect>...");
    }
    
    CreateEmbed::new()
        .author(CreateEmbedAuthor::new("Rustendo"))
        .title("Searching...")
        .description(format!("Looking for {}", query))
        .footer(CreateEmbedFooter::new(format!(">{} | {}", current_time, username)))
}


pub fn register() -> CreateCommand {
    CreateCommand::new("play")
        .description("Play music")
        .add_option(
            CreateCommandOption::new(CommandOptionType::String, "query", "Link or search term")
                .required(true)
        )
}
